<?php
namespace Elementor;
use Elementor\Icons_Manager;

//If this file is called directly, abort.

if (!defined('ABSPATH')) {
    exit;   
}

//Class Advanced Product Catalog for WooCommerce Widget.

class APCW_Widget extends Widget_Base {

    //Function for get the slug of the element name.
    public function get_name() {
        return 'advanced-product-catalog';
    }

    //Function for get the name of the element.
    public function get_title() {
        return esc_html__('Advanced Product Catalog', APCW_DOMAIN);
    }

    //Function for get the icon of the element.
    public function get_icon() {
        return ' eicon-product-meta';
    }

    //Function for include element into the category.
    public function get_categories() {
        return ['advanced-product-catalog'];
    }

    //Adding the controls fields for the Item Lists Element.

    protected function register_controls() {

        //Start Advanced Product Catalog Lists General Section.
        $this->start_controls_section(
            'apcw_product_section', array(
                'label'         => esc_html__('Item Configuration', APCW_DOMAIN),
            )
        );

        //Item Type.
        $this->add_control(
            'apcw_design', [
                'label'         => esc_html__('Catalog Type', APCW_DOMAIN),
                'type'          => Controls_Manager::SELECT,
                'options'       => [
                    'grid'          => esc_html__('Grid', APCW_DOMAIN),
                    'carousel'      => esc_html__('Carousel', APCW_DOMAIN),
                ],
                'default'       => 'grid',
            ]
        );

        //Item Grid Layout.
        $this->add_control(
            'apcw_product_grid_layout', [
                'label'         => esc_html__('Catalog Style', APCW_DOMAIN),
                'type'          => Controls_Manager::SELECT,
                'options'       => [
                    'grid-layout-1' => esc_html__('Layout 1', APCW_DOMAIN),
                    'grid-layout-2' => esc_html__('Layout 2', APCW_DOMAIN),
					'grid-layout-3' => esc_html__('Layout 3', APCW_DOMAIN),
					'grid-layout-4' => esc_html__('Layout 4', APCW_DOMAIN),
                    'grid-layout-5' => esc_html__('Layout 5', APCW_DOMAIN),
                ],
                'condition'     => ['apcw_design' => 'grid'],
                'default'       => 'grid-layout-1',
            ]
        );

       //Item Grid Layout.
        $this->add_control(
            'apcw_product_carousel_layout', [
                'label'         => esc_html__('Catalog Style', APCW_DOMAIN),
                'type'          => Controls_Manager::SELECT,
                'options'       => [
                    'carousel-layout-1' => esc_html__('Layout 1', APCW_DOMAIN),
                    'carousel-layout-2' => esc_html__('Layout 2', APCW_DOMAIN),
                    'carousel-layout-3' => esc_html__('Layout 3', APCW_DOMAIN),
                    'carousel-layout-4' => esc_html__('Layout 4', APCW_DOMAIN),
                    'carousel-layout-5' => esc_html__('Layout 5', APCW_DOMAIN),
                ],
                'condition'     => ['apcw_design' => 'carousel'],
                'default'       => 'carousel-layout-1',
            ]
        );

        //Grid Columns.
        $this->add_responsive_control(
            'apcw_grid_columns', [
                'label'             => esc_html__('Grid Columns', APCW_DOMAIN),
                'type'              => Controls_Manager::SELECT,
                'default'           => '3',
                'tablet_default'    => '2',
                'mobile_default'    => '1',
                'options' => [
                    '1' => '1',
                    '2' => '2',
                    '3' => '3',
                    '4' => '4',
                    '5' => '5',
                    '6' => '6',
                ],  
                
                'prefix_class'      => 'elementor-grid%s-',
                'frontend_available'=> true,
                'selectors'         => [
                    '.elementor-msie {{WRAPPER}} .elementor-product-item' => 'width: calc( 100% / {{SIZE}} )',
                ],
                'condition'     => ['apcw_design' => 'grid'],
            ]
        );

        //No.of Post.
        $this->add_control(
            'apcw_number_of_posts', [
                'label'         => esc_html__( 'Display No. of Product', APCW_DOMAIN ),
				'type'          => Controls_Manager::NUMBER,
				'min'           => 1,
				'max'           => 100,
				'step'          => 1,
				'default'       => 10,
            ]
        );

        //Show Image.
        $this->add_control(
            'apcw_show_image', [
                'label'         => esc_html__('Show Image', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__('Show', APCW_DOMAIN),
                'label_off'     => esc_html__('Hide', APCW_DOMAIN),
                'default'       => 'yes',
                'separator'     => 'before',
            ]
        );

        //Image Size.
        $this->add_group_control(
            Group_Control_Image_Size::get_type(), [
                'name'          => 'apcw_product_image_size',
                'exclude'       => ['custom'],
                'default'       => 'medium',
                'condition'     => ['apcw_show_image' => 'yes'],
            ]
        );

        //Show Title.
        $this->add_control(
            'apcw_show_title', [
                'label'         => esc_html__('Show Title', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_off'     => esc_html__('Show', APCW_DOMAIN),
                'label_on'      => esc_html__('Hide', APCW_DOMAIN),
                'separator'     => 'before',
                'default'       => 'yes',
            ]
        );

        //Title tag.
        $this->add_control(
            'apcw_title_tag', [
                'label'         => esc_html__('Title HTML Tag', APCW_DOMAIN),
                'type'          => Controls_Manager::SELECT,
                'options'       => [
                    'h1'        => 'H1',
                    'h2'        => 'H2',
                    'h3'        => 'H3',
                    'h4'        => 'H4',
                    'h5'        => 'H5',
                    'h6'        => 'H6',
                    'div'       => 'div',
                    'span'      => 'span',
                    'p'         => 'p',
                ],
                'condition'     => ['apcw_show_title' => 'yes'],
                'default'       => 'h2',
            ]
        );

        //Show Description.
        $this->add_control(
            'apcw_show_description', [
                'label'         => esc_html__('Show Description', APCW_DOMAIN),
                'type'          => Controls_Manager::SELECT,
                'options'       => [
                    'hide'      => esc_html__('Hide', APCW_DOMAIN),
                    'short'     => esc_html__('Short', APCW_DOMAIN),
					'full'      => esc_html__('Full', APCW_DOMAIN),
                ],
                'default'       => 'full',
            ]
        );

        //Content Length.
        $this->add_control(
            'apcw_content_length', [
                'label'         => esc_html__('Content Length', APCW_DOMAIN),
                'type'          => Controls_Manager::NUMBER,
                'default'       => apply_filters('apcw_content_length', 10),
            ]
        );

        //Read More Button.
        $this->add_control(
            'apcw_show_read_more', [
                'label'         => esc_html__('Read More', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__('Show', APCW_DOMAIN),
                'label_off'     => esc_html__('Hide', APCW_DOMAIN),
                'default'       => 'no',
                'separator'     => 'before',
                'condition'     => ['apcw_show_description' => 'full'],
            ]
        );

        //Read More Text.
        $this->add_control(
			'apcw_read_more_text', [
				'label'         => esc_html__('Read More Text', APCW_DOMAIN),
				'type'          => Controls_Manager::TEXT,
				'default'       => esc_html__('Read More »', APCW_DOMAIN),
                'condition'     => ['apcw_show_read_more' => 'yes'],
			]
		);

        //Show Category.
        $this->add_control(
            'apcw_show_category', [
                'label'         => esc_html__('Show Category', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__('Show', APCW_DOMAIN),
                'label_off'     => esc_html__('Hide', APCW_DOMAIN),
                'default'       => 'yes',
                'separator'     => 'before',
        ]
        );

        //Category Position.
        $this->add_control(
            'apcw_category_position', [
                'label'         => esc_html__('Category Position', APCW_DOMAIN),
                'type'          => Controls_Manager::SELECT,
                'options'       => [
                    'above_title'   => esc_html__('Above title', APCW_DOMAIN),
					'under_title'   => esc_html__('Under title', APCW_DOMAIN),
                ],
                'condition'     => ['apcw_show_category' => 'yes'],
                'default'       => 'above_title',
            ]
        );

        //Show Tags.
        $this->add_control(
            'apcw_show_tag', [
                'label'         => esc_html__('Show Tag', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__('Show', APCW_DOMAIN),
                'label_off'     => esc_html__('Hide', APCW_DOMAIN),
                'default'       => 'no',
            ]
        );

        //Tag Position.
        $this->add_control(
            'apcw_tag_position', [
                'label'         => esc_html__('Tag Position', APCW_DOMAIN),
                'type'          => Controls_Manager::SELECT,
                'options'       => [
                    'above_title'   => esc_html__('Above title', APCW_DOMAIN),
					'under_title'   => esc_html__('Under title', APCW_DOMAIN),
                ],
                'condition'     => ['apcw_show_tag' => 'yes'],
                'default'       => 'above_title',
            ]
        );

        //Show Sale Price.
        $this->add_control(
            'apcw_show_sale_price', [
                'label'         => esc_html__('Show Sale Price', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__('Show', APCW_DOMAIN),
                'label_off'     => esc_html__('Hide', APCW_DOMAIN),
                'default'       => 'yes',
            ]
        );

        //Show Regular Price.
        $this->add_control(
            'apcw_show_regular_price', [
                'label'         => esc_html__('Show Regular Price', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__('Show', APCW_DOMAIN),
                'label_off'     => esc_html__('Hide', APCW_DOMAIN),
                'default'       => 'yes',
            ]
        );

        //Show Rating.
        $this->add_control(
            'apcw_show_rating', [
                'label'         => esc_html__('Show Rating', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_off'     => esc_html__('Show', APCW_DOMAIN),
                'label_on'      => esc_html__('Hide', APCW_DOMAIN),
                'separator'     => 'before',
                'default'       => 'yes',
            ]
        );

        $this->end_controls_section();
        //End Lists General Section.

        //Start Query Configuration Section.
        $this->start_controls_section(
            'apcw_product_query', array(
                'label'         => esc_html__('Query Configuration', APCW_DOMAIN),
            )
        );

        //Product Types.
        $this->add_control(
            'apcw_product_type', [
                'label'         => esc_html__('Product Types', APCW_DOMAIN),
                'label_block'   => true,
                'type'          => Controls_Manager::SELECT2,
                'multiple'      => true,
                'options'       => apcw_get_product_types()
            ]
        );

        //Product Source.
        $this->add_control(
            'apcw_product_source', [
                'label'         => esc_html__('Product Source', APCW_DOMAIN),
                'label_block'   => true,
                'type'          => Controls_Manager::SELECT,
                'default'       =>  'select',
                'multiple'      => true,
                'options'       => [
                    'select'                => esc_html__( 'Select', APCW_DOMAIN ),
					'recent-products'       => esc_html__( 'Recent Products', APCW_DOMAIN ),
					'featured-products'     => esc_html__( 'Featured Products', APCW_DOMAIN ),
                    'best-selling-products' => esc_html__( 'Best Selling Products', APCW_DOMAIN ),
                    'sale-products'         => esc_html__( 'Sale Products', APCW_DOMAIN ),
                    'top-products'          => esc_html__( 'Top Rated Products', APCW_DOMAIN ),
				],
            ]
        );

        //Order By.
        $this->add_control(
            'apcw_product_orderby', [
                'label'         => esc_html__('Order By', APCW_DOMAIN),
                'type'          => Controls_Manager::SELECT,
                'default'       => 'date',
                'options'       => [
                    'date'          => esc_html__('Date', APCW_DOMAIN),
                    'title'         => esc_html__('Title', APCW_DOMAIN),
                    'menu_order'    => esc_html__('Menu Order', APCW_DOMAIN),
                    'rand'          => esc_html__('Random', APCW_DOMAIN),
                    'ID'            => esc_html__('Post ID', APCW_DOMAIN),
                    'author'        => esc_html__('Author', APCW_DOMAIN),
                    'rating'        => esc_html__('Rating', APCW_DOMAIN),
                ],
            ]
        );

        //Sort By.
        $this->add_control(
            'apcw_product_sort_order', [
                'label'         => esc_html__('Sort By', APCW_DOMAIN),
                'type'          => Controls_Manager::SELECT,
                'default'       => 'desc',
                'options'       => [
                    'asc'           => esc_html__('ASC', APCW_DOMAIN),
                    'desc'          => esc_html__('DESC', APCW_DOMAIN),
                ],
            ]
        );

        //Include Product Categories.
        $this->add_control(
            'apcw_product_categories', [
                'label'         => esc_html__('Product Categories', APCW_DOMAIN),
                'type'          => Controls_Manager::SELECT2,
                'label_block'   => true,
                'multiple'      => true,
                'description'   => esc_html__('Select the product categories you want to show', APCW_DOMAIN),
                'separator'     => 'before',
                'options'       => apcw_product_categories(),
            ]
        );

        //Exclude Product Categories.
        $this->add_control(
            'apcw_exclude_product_categories', [
                'label'         => esc_html__('Exclude Above Product Categories', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__('Yes', APCW_DOMAIN),
                'label_off'     => esc_html__('No', APCW_DOMAIN),
                'default'       => 'yes',
            ]
        );

        //Include Product Tags.
        $this->add_control(
            'apcw_product_tags', [
                'label'         => esc_html__('Product Tags', APCW_DOMAIN),
                'type'          => Controls_Manager::SELECT2,
                'label_block'   => true,
                'multiple'      => true,
                'description'   => esc_html__('Select the product tags you want to show', APCW_DOMAIN),
                'options'       => apcw_product_tags(),
                'separator'     => 'before',
            ]
        );

        //Exclude Tags.
        $this->add_control(
            'apcw_exclude_product_tags', [
                'label'         => esc_html__('Exclude Above Product Tags', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__('Yes', APCW_DOMAIN),
                'label_off'     => esc_html__('No', APCW_DOMAIN),
                'default'       => 'yes',
            ]
        );

        //Include Author.
        $this->add_control(
            'apcw_product_author', [
                'label'         => esc_html__('Product Author', APCW_DOMAIN),
                'type'          => Controls_Manager::SELECT2,
                'label_block'   => true,
                'multiple'      => true,
                'description'   => esc_html__('Select the product author you want to show', APCW_DOMAIN),
                'options'       => apcw_product_author(),
                'separator'     => 'before',
            ]
        );

        //Exclude Author.
        $this->add_control(
            'apcw_exclude_product_authors', [
                'label'         => esc_html__('Exclude Above Product Authors', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__('Yes', APCW_DOMAIN),
                'label_off'     => esc_html__('No', APCW_DOMAIN),
                'default'       => 'yes',
            ]
        );

        //Off Set.
        $this->add_control(
            'apcw_product_offset', [
                'label'         => esc_html__( 'Off Set', APCW_DOMAIN ),
				'type'          => Controls_Manager::NUMBER,
            ]
        );

        $this->end_controls_section();
        //End Query Configuration Section.

        //Start Grid Configuration.
        $this->start_controls_section(
            'apcw_grid_section', array(
                'label'         => esc_html__('Grid Configuration', APCW_DOMAIN),
                'condition'     => ['apcw_design' => 'grid'],
            )
        );

        //Grid Column Gap.
        $this->add_responsive_control(
            'apcw_product_column_gap', [
                'label'         => esc_html__('Grid Column Gap', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'default'       => [
                    'size'      => 30,
                ],
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'     => [
                    '{{WRAPPER}} .product-grid-gap' => 'grid-column-gap: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Grid Row Gap.
        $this->add_responsive_control(
            'apcw_product_row_gap', [
                'label'         => esc_html__('Grid Row Gap', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'default'       => [
                    'size'      => 30,
                ],
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'     => [
                    '{{WRAPPER}} .product-grid-gap' => 'grid-row-gap: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        //End Grid Configuration.

        //Start Carousel Configuration.
        $this->start_controls_section(
            'apcw_slider_options', [
                'label'         => esc_html__('Slider Configuration', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_CONTENT,
                'condition'     => ['apcw_design' => 'carousel'],
            ]
        );

        // Slide to show.
        $this->add_responsive_control(
            'apcw_slider_slides_to_show', [
                'label'             => esc_html__('Slides To Show', APCW_DOMAIN),
                'type'              => Controls_Manager::NUMBER,
                'default'           => 3,
                'devices'           => [ 'desktop', 'tablet', 'mobile' ],
				'desktop_default'   => 3,
				'tablet_default'    => 2,
				'mobile_default'    => 1,
                'frontend_available'=> true,
            ]
        );

        //Slide to Scroll.
        $this->add_control(
            'apcw_slider_slides_to_scroll', [
                'label'         => esc_html__('Slides To Scroll', APCW_DOMAIN),
                'type'          => Controls_Manager::NUMBER,
                'min'           => 1,
                'max'           => 6,
                'step'          => 1,
                'default'       => 1,
            ]
        );

        //Slider Infinite.
        $this->add_control(
            'apcw_slider_infinite', [
                'label'         => esc_html__('Show Infinite', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__( 'Show', APCW_DOMAIN ),
                'label_off'     => esc_html__( 'Hide', APCW_DOMAIN ),
                'return_value'  => 'yes',
                'default'       => 'no',
            ]
        );

        //Pause On Hover.
        $this->add_control(
            'apcw_slider_pause_on_hover', [
                'label'         => esc_html__('Pause On Hover', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__( 'Show', APCW_DOMAIN ),
                'label_off'     => esc_html__( 'Hide', APCW_DOMAIN ),
                'return_value'  => 'yes',
                'default'       => 'no',
            ]
        );

        //Slider Autoplay.
        $this->add_control(
            'apcw_slider_autoplay', [
                'label'         => esc_html__('Autoplay', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'default'       => 'yes',
                'label_off'     => esc_html__('Off', APCW_DOMAIN),
                'label_on'      => esc_html__('On', APCW_DOMAIN),
            ]
        );

        //Autoplay Speed.
        $this->add_control(
            'apcw_slider_speed', [
                'label'         => esc_html__('Autoplay Speed', APCW_DOMAIN),
                'type'          => Controls_Manager::NUMBER,
                'min'           => 1000,
                'max'           => 10000,
                'step'          => 1000,
                'default'       => 5000,
                'description'   => esc_html__('Speed is in milliseconds (1000ms = 1s).', APCW_DOMAIN)
            ]
        );

        //Show Dots.
        $this->add_control(
            'apcw_slider_dots', [
                'label'         => esc_html__('Show Dots', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__( 'Show', APCW_DOMAIN ),
                'label_off'     => esc_html__( 'Hide', APCW_DOMAIN ),
                'return_value'  => 'yes',
                'default'       => 'yes',
            ]
        );

        //Show Arrows.
        $this->add_control(
            'apcw_slider_arrows', [
                'label'         => esc_html__('Show Arrows', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__( 'Show', APCW_DOMAIN ),
                'label_off'     => esc_html__( 'Hide', APCW_DOMAIN ),
                'return_value'  => 'yes',
                'default'       => 'no',
            ]
        );

        //Previous Icon.
        $this->add_control(
            'apcw_prev_icon', [
                'label'         => esc_html__( 'Previous icon', APCW_DOMAIN ),
                'type'          => Controls_Manager::ICONS,
                'default'       => [
                    'value'     => 'fas fa-angle-left',
                    'library'   => 'solid',
                ],
            ]
        );

        //Next Icon.
        $this->add_control(
            'apcw_next_icon', [
                'label'         => esc_html__( 'Next icon', APCW_DOMAIN ),
                'type'          => Controls_Manager::ICONS,
                'default'       => [
                    'value'     => 'fas fa-angle-right',
                    'library'   => 'solid',
                ]
            ]
        );

        $this->end_controls_section(); 
        //End Carousel Configuration.

        //Start Advanced Section.
        $this->start_controls_section(
            'apcw_advanced_section', array(
                'label'         => esc_html__('Advanced', APCW_DOMAIN),
            )
        );

        //Button Label.
        $this->add_control(
            'apcw_button', [
                'label'         => esc_html__('Button', APCW_DOMAIN),
                'type'          => Controls_Manager::HEADING,
            ]   
        );

        //Show Cart Button.
        $this->add_control(
            'apcw_show_cart_button', [
                'label'         => esc_html__('Show Cart Button', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__('Show', APCW_DOMAIN),
                'label_off'     => esc_html__('Hide', APCW_DOMAIN),
                'default'       => 'yes',
                'separator'     => 'before',
            ]
        );

        //Show Custom Button.
        $this->add_control(
            'apcw_show_custom_button', [
                'label'         => esc_html__('Show Custom Button', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__('Show', APCW_DOMAIN),
                'label_off'     => esc_html__('Hide', APCW_DOMAIN),
                'default'       => 'yes',
                'condition'     => ['apcw_show_cart_button' => 'yes'],
            ]
        );

        //Simple Product Text.
        $this->add_control(
			'apcw_simple_button_text', [
				'label'         => esc_html__('Simple Product Text', APCW_DOMAIN),
				'type'          => Controls_Manager::TEXT,
				'default'       => esc_html__('Add To Cart', APCW_DOMAIN),
                'condition'     => ['apcw_show_custom_button' => 'yes'],
			]
		);

        //Grouped Product Text.
        $this->add_control(
			'apcw_grouped_button_text', [
				'label'         => esc_html__('Grouped Product Text', APCW_DOMAIN),
				'type'          => Controls_Manager::TEXT,
				'default'       => esc_html__('View Products', APCW_DOMAIN),
                'condition'     => ['apcw_show_custom_button' => 'yes'],
			]
		);

        //External/Affiliate Button Text.
        $this->add_control(
			'apcw_external_button_text', [
				'label'         => esc_html__('External/Affiliate Button Text', APCW_DOMAIN),
				'type'          => Controls_Manager::TEXT,
				'default'       => esc_html__('Buy on the WordPress swag store!', APCW_DOMAIN),
                'condition'     => ['apcw_show_custom_button' => 'yes'],
			]
		);

        //Variable Product Text.
        $this->add_control(
			'apcw_variable_button_text', [
				'label'         => esc_html__('Variable Product Text', APCW_DOMAIN),
				'type'          => Controls_Manager::TEXT,
				'default'       => esc_html__('Select Options', APCW_DOMAIN),
                'condition'     => ['apcw_show_custom_button' => 'yes'],
			]
		);

        //Sale Badge.
        $this->add_control(
            'apcw_advanced', [
                'label'         => esc_html__('Sale Badge', APCW_DOMAIN),
                'type'          => Controls_Manager::HEADING,
                'separator'     => 'before',
            ]
        );

        //Show Sale Badge.
        $this->add_control(
            'apcw_show_badge', [
                'label'         => esc_html__('Sale Badge', APCW_DOMAIN),
                'type'          => Controls_Manager::SWITCHER,
                'label_on'      => esc_html__('Show', APCW_DOMAIN),
                'label_off'     => esc_html__('Hide', APCW_DOMAIN),
                'default'       => 'yes',
                'separator'     => 'before',
            ]
        );

        //Sale Badge Text.
        $this->add_control(
			'apcw_sale_badge', [
				'label'         => esc_html__( 'Sale Badge Text', APCW_DOMAIN ),
				'type'          => Controls_Manager::TEXT,
				'default'       => esc_html__( 'Sale', APCW_DOMAIN ),
                'condition'     => ['apcw_show_badge' => 'yes'],
			]
		);

        $this->end_controls_section();
        //End Grid Configuration.

        //Style Section Start.
        //Start Item Style.
        $this->start_controls_section(
            'apcw_item_style', [
                'label'         => esc_html__('Item', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_STYLE,
            ]
        );

        //Item border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'          => 'apcw_item_border',
                'selector'      => '{{WRAPPER}} .apcw-layout-1 .apcw-item-container,
                    {{WRAPPER}} .apcw-layout-2 .apcw-item-container,
                    {{WRAPPER}} .apcw-layout-3 .apcw-item-container,
                    {{WRAPPER}} .apcw-layout-4 .apcw-item-container',
                    'conditions' => [
                    'terms'      => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-1','grid-layout-2','grid-layout-3','grid-layout-4'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-1', 'carousel-layout-2', 'carousel-layout-3', 'carousel-layout-4'],
                        ],
                    ],
                ],
            ]
        );

        //Item Height.
        $this->add_control(
            'apcw_image_flip_height', [
                'label'         => esc_html__('Item Box Height', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'size_units'    => [ 'px' ],
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 400,
                        'step'  => 1,
                    ], 
                ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-layout-5 img,
                    {{WRAPPER}} .apcw-layout-5 .apcw-item-container' => 'height: {{SIZE}}{{UNIT}};',
                ],
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-5'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-5'],
                        ],
                    ],
                ],
            ]
        );

        //Item Tab.
        $this->start_controls_tabs('apcw_item_tabs');

        //Start Item Normal Tab.
        $this->start_controls_tab('apcw_item_normal', [
            'label'         => esc_html__('Normal', APCW_DOMAIN),
            ]
        );

        //Item Box Shadow.
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(), [
                'name'          => 'apcw_box_shadow',
                'selector'      => '{{WRAPPER}} .apcw-layout-1 .apcw-item-container,
                {{WRAPPER}} .apcw-layout-2 .apcw-item-container,
                {{WRAPPER}} .apcw-layout-3 .apcw-item-container,
                {{WRAPPER}} .apcw-layout-4 .apcw-item-container,
                {{WRAPPER}} .apcw-layout-5 .apcw-item-container',
            ]
        );

        //Item Content Background Color.
        $this->add_control(
            'apcw_box_bg_color', [
                'label'         => esc_html__('Content Background', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-layout-1 .apcw-content-container,
                    {{WRAPPER}} .apcw-layout-3 .apcw-content-container,
                    {{WRAPPER}} .apcw-layout-4 .apcw-content-container,
                    {{WRAPPER}} .apcw-layout-5 .apcw-content-container' => 'background-color: {{VALUE}};',
                ],
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-1','grid-layout-3','grid-layout-4','grid-layout-5'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-1','carousel-layout-3','carousel-layout-4','carousel-layout-5'],
                        ],
                    ],
                ],
            ]
        );

        $this->end_controls_tab();
        //End Item Normal Tab.

        //Start Item Hover Tab.
        $this->start_controls_tab('apcw_item_hover', [
            'label'             => esc_html__('Hover', APCW_DOMAIN),
            ]
        );

        //Item Box Shadow.
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(), [
                'name'          => 'apcw_box_hover_shadow',
                'selector'      => '{{WRAPPER}} .apcw-layout-1 .apcw-item-container:hover,
                {{WRAPPER}} .apcw-layout-2 .apcw-item-container:hover,
                {{WRAPPER}} .apcw-layout-3 .apcw-item-container:hover,
                {{WRAPPER}} .apcw-layout-4 .apcw-item-container:hover,
                {{WRAPPER}} .apcw-layout-5 .apcw-item-container:hover',
            ]
        );

        //Item Content Background Color.
        $this->add_control(
            'apcw_box_bg_hover_color', [
                'label'         => esc_html__('Content Background Hover', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-layout-1:hover .apcw-content-container,
                    {{WRAPPER}} .apcw-layout-5:hover .apcw-content-container,
                    {{WRAPPER}} .apcw-layout-3:hover .apcw-content-container,
                    {{WRAPPER}} .apcw-layout-4:hover .apcw-content-container' => 'background-color: {{VALUE}};',
                ],
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-1','grid-layout-3','grid-layout-4','grid-layout-5'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-1','carousel-layout-3','carousel-layout-4','carousel-layout-5'],
                        ],
                    ],
                ],
            ]
        );

        //Item Border Color.
        $this->add_control(
            'apcw_box_border_hover_color', [
                'label'         => esc_html__('Border Hover Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'separator'     => 'after',
                'selectors'     => [
                    '{{WRAPPER}} .apcw-layout-1 .apcw-item-container:hover,
                    {{WRAPPER}} .apcw-layout-2 .apcw-item-container:hover,
                    {{WRAPPER}} .apcw-layout-3 .apcw-item-container:hover,
                    {{WRAPPER}} .apcw-layout-4 .apcw-item-container:hover,
                    {{WRAPPER}} .apcw-layout-5 .apcw-item-container:hover' => 'border-color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();
        //End Item Hover Tab.

        $this->end_controls_tabs();
        //End Item Tab.

        //Item Border Radius.
        $this->add_responsive_control(
            'apcw_item_border_radius', [
                'label'         => esc_html__('Item Border Radius', APCW_DOMAIN),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => ['px', '%'],
                'default'       => [
                    'top'       => '10',
                    'right'     => '10',
                    'bottom'    => '10',
                    'left'      => '10',
                    'isLinked'  => true,
                ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-layout-1 .apcw-item-container,
                    {{WRAPPER}} .apcw-layout-2 .apcw-item-container,
                    {{WRAPPER}} .apcw-layout-3 .apcw-item-container,
                    {{WRAPPER}} .apcw-layout-4 .apcw-item-container,
                    {{WRAPPER}} .apcw-layout-4 .apcw-layout-img img' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-1','grid-layout-3','grid-layout-4','grid-layout-2'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-1','carousel-layout-3','carousel-layout-4','carousel-layout-2'],
                        ],
                    ],
                ],           
            ]
        );

        //Item Border Radius.
        $this->add_responsive_control(
            'apcw_flip_border_radius',
            [
                'label'         => esc_html__('Item Border Radius', APCW_DOMAIN),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => [ 'px', '%' ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-layout-5 .apcw-flip-front,
                    {{WRAPPER}} .apcw-layout-5 .apcw-content-container,
                    {{WRAPPER}} .apcw-layout-5 .apcw-layout-img img' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'conditions'        => [
                    'terms'         => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-5'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-5'],
                        ],
                    ],
                ],
            ]
        );

        //Content Padding.
        $this->add_responsive_control(
            'apcw_content_padding', [
                'label'         => esc_html__('Content Padding', APCW_DOMAIN),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => ['px', '%'],
                'default'       => [
                    'top'       => '20',
                    'right'     => '20',
                    'bottom'    => '20',
                    'left'      => '20',
                    'isLinked'  => true,
                ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-layout-1 .apcw-content-container,
                    {{WRAPPER}} .apcw-layout-2 .apcw-content-container,
                    {{WRAPPER}} .apcw-layout-3 .apcw-content-container,
                    {{WRAPPER}} .apcw-layout-4 .apcw-content-container,
                    {{WRAPPER}} .apcw-layout-5 .apcw-content-container' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        //Content Alignment.
        $this->add_responsive_control(
            'apcw_content_align', [
                'label'         => esc_html__('Content Alignment', APCW_DOMAIN),
                'type'          => Controls_Manager::CHOOSE,
                'options'       => [
                    'left'      => [
                        'title' => esc_html__('Left', APCW_DOMAIN),
                        'icon'  => 'eicon-text-align-left',
                    ],
                    'center'    => [
                        'title' => esc_html__('Center', APCW_DOMAIN),
                        'icon'  => 'eicon-text-align-center',
                    ],
                    'right'     => [
                        'title' => esc_html__('Right', APCW_DOMAIN),
                        'icon'  => 'eicon-text-align-right',
                    ],
                ],
                'default'       => 'center',
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-content-container' => 'text-align: {{VALUE}};',
                ],
            ]
        );

        //Gradient Background.
        $this->add_group_control(
            Group_Control_Background::get_type(),[
                'name'          => 'icon_background_color_gradient',
                'selector'      => '{{WRAPPER}} .apcw-layout-4 .apcw-content-container',
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-4'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-4'],
                        ],
                    ],
                ],
                'types'             => ['gradient'],
                'exclude'           => ['image','classic'],
                'fields_options'    => [
                    'background'    => [
                        'label'     => esc_html__('Content Background Color', APCW_DOMAIN),
                    ],
                ]
            ]
        );

        $this->end_controls_section();
        //End Box Style.

        //Start Image Style.
        $this->start_controls_section(
            'apcw_image_style', [
                'label'         => esc_html__('Image', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_STYLE,
                'condition'     => ['apcw_show_image' => 'yes'],
            ]
        );

        //Image Height.
        $this->add_control(
            'apcw_image_height', [
                'label'         => esc_html__('Height', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'size_units'    => [ 'px' ],
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 500,
                        'step'  => 1,
                    ],
                ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-layout-img img' => 'height: {{SIZE}}{{UNIT}};',
                ],
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-1', 'grid-layout-2', 'grid-layout-3'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-1', 'carousel-layout-2', 'carousel-layout-3'],
                        ],
                    ],
                ],
            ]
        );

        //Image Width.
        $this->add_control(
            'apcw_image_width', [
                'label'         => esc_html__('Width', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'size_units'    => [ 'px', '%' ],
                'range'         => [
					'px'        => [
						'min'   => 0,
						'max'   => 500,
						'step'  => 1,
					],
					'%'         => [
						'min'   => 0,
						'max'   => 100,
					],
				],
				'default'       => [
					'unit'      => '%',
					'size'      => 100,
				],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-layout-img img' => 'width: {{SIZE}}{{UNIT}};',
                ],
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-1', 'grid-layout-3', 'grid-layout-5'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-1', 'carousel-layout-3', 'carousel-layout-5'],
                        ],
                    ],
                ],
            ]
        );

        //Image Background Color.
        $this->add_control(
            'apcw_flip_img_color', [
                'label'         => esc_html__('Image Background Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-layout-5 .apcw-flip-front,
                    {{WRAPPER}} .apcw-layout-4 .apcw-layout-img' => 'background-color: {{VALUE}};',
                ],
                'default'       =>'#CCD2EF',
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-4', 'grid-layout-5'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-4', 'carousel-layout-5'],
                        ],
                    ],
                ],
            ]
        );

        //Image Overlay Height.
        $this->add_control(
            'apcw_img_overlay', [
                'label'         => esc_html__('Image Overlay Height', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'size_units'    => [ 'px', '%' ],
                'range'         => [
					'%'         => [
						'min'   => 0,
						'max'   => 100,
					],
                    'px'        => [
						'min'   => 0,
						'max'   => 500,
						'step'  => 1,
					],
				],
				'default'       => [
					'unit'      => '%',
					'size'      => 80,
				],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-layout-2 .apcw-item-container:hover .apcw_overlay' => 'height: {{SIZE}}{{UNIT}};',
                ],
                'condition'     => ['apcw_product_grid_layout' => 'grid-layout-2'],
            ]
        );

        //Image Border Radius.
        $this->add_responsive_control(
            'apcw_image_border_radius',
            [
                'label'         => esc_html__('Border Radius', APCW_DOMAIN),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => [ 'px', '%' ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-layout-1 .apcw-layout-img img,
                    {{WRAPPER}} .apcw-layout-3 .apcw-layout-img img' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-1', 'grid-layout-3'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-1', 'carousel-layout-3'],
                        ],
                    ],
                ],
            ]
        );

        //Image Overlay Color.
        $this->add_control(
            'apcw_image_overlay_color', [
                'label'         => esc_html__('Image Overlay Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-layout-2 .apcw_overlay' => 'background-color: {{VALUE}};',
                ],
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-2'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-2'],
                        ],
                    ],
                ],
            ]
        );

        //Image Tab.
        $this->start_controls_tabs('apcw_image_effect_tabs');

        //Start Image Normal Tab.
        $this->start_controls_tab('apcw_normal_img', [
            'label'         => esc_html__('Normal', APCW_DOMAIN),
            ]
        );

        //Image Css Filter.
        $this->add_group_control(
            Group_Control_Css_Filter::get_type(), [
            'name'          => 'apcw_image_filters',
            'selector'      => '{{WRAPPER}} .apcw-item-container .apcw-layout-img',
            ]
        );

        $this->end_controls_tab();
        //End Image Normal Tab.

        //Start Image Hover Tab.
        $this->start_controls_tab('apcw_hover_img', [
            'label'         => esc_html__('Hover', APCW_DOMAIN),
            ]
        );

        //Image Css Filter Hover.
        $this->add_group_control(
            Group_Control_Css_Filter::get_type(), [
            'name'          => 'apcw_image_hover_filters',
            'selector'      => '{{WRAPPER}} .apcw-item-container:hover .apcw-layout-img img',
            ]
        );

        $this->end_controls_tab();
        //End Image Hover Tab.

        $this->end_controls_tabs();
        //End Image Tab.

        //Image Alignment.
        $this->add_responsive_control(
            'apcw_image_align', [
                'label'         => esc_html__('Image Alignment', APCW_DOMAIN),
                'type'          => Controls_Manager::CHOOSE,
                'options'       => [
                    'left'      => [
                        'title' => esc_html__('Left', APCW_DOMAIN),
                        'icon'  => 'eicon-text-align-left',
                    ],
                    'center'    => [
                        'title' => esc_html__('Center', APCW_DOMAIN),
                        'icon'  => 'eicon-text-align-center',
                    ],
                    'right'     => [
                        'title' => esc_html__('Right', APCW_DOMAIN),
                        'icon'  => 'eicon-text-align-right',
                    ],
                ],
                'default'       => 'center',
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container a.apcw-img' => 'justify-content: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_section();
        //End Image Style.

        //Start Title Style.
        $this->start_controls_section(
            'apcw_title_style', [
                'label'         => esc_html__('Title', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_STYLE,
                'condition'         => ['apcw_show_title' => 'yes'],
            ]
        );

        //Title Spacing.
        $this->add_control(
            'apcw_title_spacing', [
                'label'         => esc_html__('Title Spacing', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 100,
                    ],
                ],
                'default'       => [
					'unit'      => 'px',
					'size'      => 10,
				],
                'frontend_available' => true,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-layout-title-tag' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Title Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'          => 'title_typography',
                'label'         => esc_html__('Title Typography', APCW_DOMAIN),
                'selector'      => '{{WRAPPER}} .apcw-item-container .apcw-layout-title-tag',
            ]
        );

        //Title Color.
        $this->add_control(
            'apcw_title_color', [
                'label'         => esc_html__('Title Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-layout-title-tag' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_section();
        //End Title Style.

        //Start Description Style.
        $this->start_controls_section(
            'apcw_description_style', [
                'label'         => esc_html__('Content', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_STYLE,
            ]
        );

        //Description Spacing.
        $this->add_control(
            'apcw_description_spacing', [
                'label'         => esc_html__('Description Spacing', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 100,
                    ],
                ],
                'default'       => [
					'unit'      => 'px',
					'size'      => 10,
				],
                'frontend_available' => true,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-product-description' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Description Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'          => 'apcw_description_typography',
                'label'         => esc_html__('Description Typography', APCW_DOMAIN),
                'selector'      => '{{WRAPPER}} .apcw-item-container .apcw-product-description',
            ]
        );

        //Description Color
        $this->add_control(
            'apcw_description_color', [
                'label'         => esc_html__('Description Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-product-description' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Read More Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'          => 'apcw_read_more_typography',
                'label'         => esc_html__('Read More Typography', APCW_DOMAIN),
                'selector'      => '{{WRAPPER}} .apcw-item-container a.apcw_product_read-more',
            ]
        );

        //Start Read More Tab.
        $this->start_controls_tabs('apcw_read_color_tabs');

        //Start Normal Read More Tab.
        $this->start_controls_tab('apcw_read_normal', [
            'label'             => esc_html__('Normal', APCW_DOMAIN),
            ]
        );

        //Read More Color.
        $this->add_control( 
            'apcw_read_more_color', [
                'label'         => esc_html__('Read More Text Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'default'       => '#000000',
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container a.apcw_product_read-more' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();
        //End Normal Read More Tab.

        //Start Hover Read More Tab.
        $this->start_controls_tab('apcw_read_hover', [
            'label'             => esc_html__('Hover', APCW_DOMAIN),
            ]
        );

        //Read More Hover Color.
        $this->add_control(
            'apcw_read_more_color_hover', [
                'label'         => esc_html__('Read More Hover Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container a.apcw_product_read-more:hover' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();
        //End Hover Read More Tab.

        $this->end_controls_tabs();
        //End Read More Tab.

        $this->end_controls_section();
        //End Content Style.

        //Start Category Style.
        $this->start_controls_section(
            'apcw_category_style', [
                'label'         => esc_html__('Category', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_STYLE,
                'condition'     => ['apcw_show_category' => 'yes'],
            ]
        );

        //Category Spacing.
        $this->add_control(
            'apcw_category_spacing', [
                'label'         => esc_html__('Category Spacing', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 100,
                    ],
                ],
                'default'       => [
					'unit'      => 'px',
					'size'      => 10,
				],
                'frontend_available' => true,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-layout-categories' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Category Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'          => 'category_typography',
                'label'         => esc_html__('Category Typography', APCW_DOMAIN),
                'selector'      => '{{WRAPPER}} .apcw-item-container .apcw-category-link,
                {{WRAPPER}} .apcw-item-container .apcw-category-names',
            ]
        );

        //Category Color Tab.
        $this->start_controls_tabs('apcw_category_tabs');

        //Start Category Normal Tab.
        $this->start_controls_tab('apcw_category_normal', [
            'label'             => esc_html__('Normal', APCW_DOMAIN),
            ]
        );

        //Category Color.
        $this->add_control(
            'apcw_category_color', [
                'label'         => esc_html__('Category Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-category-link,
                    {{WRAPPER}} .apcw-item-container .apcw-category-names' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();
        //End Category Normal Tab.

        //Start Category Hover Tab.
        $this->start_controls_tab('apcw_category_hover', [
            'label'             => esc_html__('Hover', APCW_DOMAIN),
            ]
        );

        //Category Hover Color.
        $this->add_control(
            'apcw_category_hover_color', [
                'label'         => esc_html__('Category Hover Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-category-link:hover,
                    {{WRAPPER}} .apcw-item-container .apcw-category-names:hover' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();
        //End Category Hover Tab.

        $this->end_controls_tabs();
        //End Category Tab.

        $this->end_controls_section();
        //End Category Style.

        //Start Tag Style.
        $this->start_controls_section(
            'apcw_tag_style', [
                'label'         => esc_html__('Tag', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_STYLE,
                'condition'     => ['apcw_show_tag' => 'yes'],
            ]
        );

        //Tag Spacing.
        $this->add_control(
            'apcw_tag_spacing', [
                'label'         => esc_html__('Tag Spacing', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 100,
                    ],
                ],
                'default'       => [
					'unit'      => 'px',
					'size'      => 10,
				],
                'frontend_available' => true,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-layout-tags' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Tag Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'          => 'apcw_tag_typography',
                'label'         => esc_html__('Tag Typography', APCW_DOMAIN),
                'selector'      => '{{WRAPPER}} .apcw-item-container .apcw-tag-link,
                {{WRAPPER}} .apcw-item-container .apcw-tag-names',
            ]
        );

        //Tag Color Tab.
        $this->start_controls_tabs('apcw_tag_tabs');

        //Start Tag Normal Tab.
        $this->start_controls_tab('apcw_tag_normal', [
            'label'             => esc_html__('Normal', APCW_DOMAIN),
            ]
        );

        //Tag Color.
        $this->add_control(
            'apcw_tag_color', [
                'label'         => esc_html__('Tag Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-tag-link,
                    {{WRAPPER}} .apcw-item-container .apcw-tag-names' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();
        //End Tag Normal Tab.

        //Start Tag Hover Tab.
        $this->start_controls_tab('apcw_tag_hover', [
            'label'             => esc_html__('Hover', APCW_DOMAIN),
                ]
        );

        //Tag Hover Color.
        $this->add_control(
            'apcw_tag_hover_color', [
                'label'         => esc_html__('Tag Hover Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-tag-link:hover,
                    {{WRAPPER}} .apcw-item-container .apcw-tag-names:hover' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();
        //End Tag Hover Tab.

        $this->end_controls_tabs();
        //End Tag Tab.

        $this->end_controls_section();
        //End Tag Style.

        //Start Prices Style.
        $this->start_controls_section(
            'apcw_prices_style', [
                'label'         => esc_html__('Prices', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_STYLE,
            ]
        );

        //Price Spacing.
        $this->add_control(
            'apcw_price_spacing', [
                'label'         => esc_html__('Price Spacing', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 100,
                    ],
                ],
                'default'       => [
					'unit'      => 'px',
					'size'      => 10,
				],
                'frontend_available' => true,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-price' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Space Between Price.
        $this->add_control(
            'apcw_space_between_prices', [
                'label'         => esc_html__('Space Between Price', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'size_units'    => [ 'px' ],
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 50,
                        'step'  => 1,
                    ],
                ],
                'default'       => [
                    'unit'      => 'px',
                    'size'      => 5,
                ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-sale-price' => 'margin-left: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Regular Price Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'          => 'apcw_regular_typography',
                'label'         => esc_html__('Regular Price Typography', APCW_DOMAIN),
                'selector'      => '{{WRAPPER}} .apcw-item-container .apcw-regular-price',
            ]
        );

        //Regular Price Color
        $this->add_control(
            'apcw_regular_color', [
                'label'         => esc_html__('Regular Price Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-regular-price' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Sale Price Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'          => 'apcw_sale_typography',
                'label'         => esc_html__('Sale Price Typography', APCW_DOMAIN),
                'selector'      => '{{WRAPPER}} .apcw-item-container .apcw-sale-price,
                {{WRAPPER}} .apcw-item-container .apcw-price-variation',
            ]
        );

        //Sale price Color.
        $this->add_control(
            'apcw_sale_color', [
                'label'         => esc_html__('Sale price Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-sale-price,
                    {{WRAPPER}} .apcw-item-container .apcw-price-variation' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_section();
        //End Prices Style.

        //Start Button Style.
        $this->start_controls_section(
            'apcw_button_style', [
                'label'         => esc_html__('Button', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_STYLE,
            ]
        );

        //Button Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'          => 'apcw_button_typography',
                'label'         => esc_html__('Typography', APCW_DOMAIN),
                'selector'      => '{{WRAPPER}} .apcw-item-container .apcw-add-cart a',
            ]
        );

        //Button Alignment.
        $this->add_responsive_control(
            'apcw_button_align', [
                'label'         => esc_html__('Alignment', APCW_DOMAIN),
                'type'          => Controls_Manager::CHOOSE,
                'options'       => [
                    'left'      => [
                        'title' => esc_html__('Left', APCW_DOMAIN),
                        'icon'  => 'eicon-text-align-left',
                    ],
                    'center'    => [
                        'title' => esc_html__('Center', APCW_DOMAIN),
                        'icon'  => 'eicon-text-align-center',
                    ],
                    'right'     => [
                        'title' => esc_html__('Right', APCW_DOMAIN),
                        'icon'  => 'eicon-text-align-right',
                    ],
                ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-button' => 'text-align: {{VALUE}};',
                ],
                'condition'     => ['apcw_product_grid_layout' => ['grid-layout-1', 'grid-layout-2','grid-layout-4', 'grid-layout-5'],],
            ]
        );

        //Button Spacing.
        $this->add_control(
            'apcw_button_spacing', [
                'label'         => esc_html__('Button Spacing', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'default'       => [
                    'size'      => 15,
                ],
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-button' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
                'condition'     => ['apcw_product_grid_layout' => ['grid-layout-1', 'grid-layout-2','grid-layout-4', 'grid-layout-5'],],
            ]
        );

        //Start Button Tab.
        $this->start_controls_tabs('apcw_button_color_tabs');

        //Start Normal Button Tab.
        $this->start_controls_tab('apcw_normal_button', [
            'label'             => esc_html__('Normal', APCW_DOMAIN),
            ]
        );

        //Button Text Color.
        $this->add_control(
            'apcw_button_color', [
                'label'         => esc_html__('Text Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-add-cart a' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Button Background Color.
        $this->add_control(
            'apcw_button_bg_color', [
                'label'         => esc_html__('Button Background Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-add-cart' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();
        //End Normal Button Tab.

        //Start Hover Button Tab.
        $this->start_controls_tab('apcw_hover_button', [
            'label' => esc_html__('Hover', APCW_DOMAIN),
            ]
        );

        //Button Text Color.
        $this->add_control(
            'apcw_button_color_hover', [
                'label'         => esc_html__('Text Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-add-cart:hover a' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Button Background Color.
        $this->add_control(
            'apcw_button_bg_color_hover', [
                'label'         => esc_html__('Button Background Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-add-cart:hover' => 'background-color: {{VALUE}};',
                ],
            ]
        );

        //Cart Button border.
        $this->add_control(
            'apcw_border_color_hover', [
                'label'         => esc_html__('Border Hover Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-add-cart:hover' => 'border-color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_tab();
        //End Hover Button Tab.

        $this->end_controls_tabs();
        //End Button Tab.

        //Cart Button border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'          => 'apcw_cart_button_border',
                'selector'      => '{{WRAPPER}} .apcw-item-container .apcw-add-cart',
            ]
        );

        //Cart Button Border Radius.
        $this->add_responsive_control(
            'apcw_border_radius', [
                'label'         => esc_html__('Border Radius', APCW_DOMAIN),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => [ 'px', '%' ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-add-cart' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        //Cart Button Border Radius.
        $this->add_responsive_control(
            'apcw_button_padding', [
                'label'         => esc_html__('Button Padding', APCW_DOMAIN),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => [ 'px', '%' ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-add-cart' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        //End Button Style.

        //Start Rating Style.
        $this->start_controls_section(
            'apcw_rating_style', [
                'label'         => esc_html__('Rating', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_STYLE,
            ]
        );

        //Rating Size.
        $this->add_responsive_control(
            'apcw_rating_size', [
                'label'         => esc_html__('Rating Size', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 100,
                    ],
                ],
                'default'       => [ 'size'  => 16 ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-star-rating' => 'font-size: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Rating Spacing.
        $this->add_control(
            'apcw_rating_spacing', [
                'label'         => esc_html__('Rating Spacing', APCW_DOMAIN),
                'type'          => Controls_Manager::SLIDER,
                'default'       => [
                    'size'      => 10,
                ],
                'range'         => [
                    'px'        => [
                        'min'   => 0,
                        'max'   => 100,
                    ],
                ],
                'frontend_available' => true,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-star-rating' => 'margin-top: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        //Rating Color.
        $this->add_control(
            'apcw_rating_color', [
                'label'         => esc_html__('Rating Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-star-rating::before' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Rating Fill Color.
        $this->add_control(
            'apcw_rating_fill_color', [
                'label'         => esc_html__('Rating Fill Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-star-rating span' => 'color: {{VALUE}};',
                ],
            ]
        );

        $this->end_controls_section();
        //End Rating Style.

        //Start Sale Badge Style.
        $this->start_controls_section(
            'apcw_sale_badge_style', [
                'label'         => esc_html__('Sale Badge', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_STYLE,
                'condition'     => ['apcw_show_badge' => 'yes'],
            ]
        );

        //Sale Badge Typography.
        $this->add_group_control(
            Group_Control_Typography::get_type(), [
                'name'          => 'apcw_sale_badge_typography',
                'label'         => esc_html__('Sale Badge Typography', APCW_DOMAIN),
                'selector'      => '{{WRAPPER}} .apcw-item-container .apcw-ribbon,
                {{WRAPPER}} .apcw-layout-2 .apcw-ribbon',
            ]
        );

        //Sale Badge Color.
        $this->add_control(
            'apcw_sale_badge_color', [
                'label'         => esc_html__('Sale Badge Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-ribbon,
                    {{WRAPPER}} .apcw-layout-2 .apcw-ribbon' => 'color: {{VALUE}};',
                ],
            ]
        );

        //Sale Badge Background Color.
        $this->add_control(
            'apcw_sale_badge_bg_color', [
                'label'         => esc_html__('Sale Badge Background Color', APCW_DOMAIN),
                'type'          => Controls_Manager::COLOR,
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-ribbon,
                    {{WRAPPER}} .apcw-layout-2 .apcw-ribbon,
                    {{WRAPPER}} .apcw-layout-3 .apcw-ribbon,
                    {{WRAPPER}} .apcw-layout-5 .apcw-ribbon' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} .apcw-layout-2 .apcw-label::before,
                    {{WRAPPER}} .apcw-layout-2 .apcw-label::after,
                    {{WRAPPER}} .apcw-layout-3 .apcw-ribbon:before' => 'border-color: {{VALUE}};',
                    '{{WRAPPER}} .apcw-layout-4 .apcw-label:after' => 'border-top-color: {{VALUE}};',
                ],
            ]
        );

        //Sale Badge Border Radius.
        $this->add_responsive_control(
            'apcw_sale_badge_border_radius', [
                'label'         => esc_html__('Border Radius', APCW_DOMAIN),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => [ 'px', '%' ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-layout-5 .apcw-ribbon' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-5'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-5'],
                        ],
                    ],
                ],
            ]
        );

        //Sale Badge Padding.
        $this->add_responsive_control(
            'apcw_sale_badge_padding', [
                'label'         => esc_html__('Padding', APCW_DOMAIN),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => [ 'px', '%' ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-item-container .apcw-ribbon' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-1','grid-layout-3','grid-layout-4','grid-layout-5'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-1','carousel-layout-3','carousel-layout-4','carousel-layout-5'],
                        ],
                    ],
                ],
            ]
        );

        //Sale Badge Shadow.
        $this->add_group_control(
            Group_Control_Box_Shadow::get_type(), [
                'name'          => 'apcw_sale_badge_shadow',
                'selector'      => '{{WRAPPER}} .apcw-layout-2 .apcw-ribbon,
                {{WRAPPER}} .apcw-layout-3 .apcw-ribbon,
                {{WRAPPER}} .apcw-layout-4 .apcw-ribbon,
                {{WRAPPER}} .apcw-layout-5 .apcw-ribbon',
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-2','grid-layout-3','grid-layout-4', 'grid-layout-5'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-2','carousel-layout-3','carousel-layout-4','carousel-layout-5'],
                        ],
                    ],
                ],
                ]
        );

        //Sale Badge Border.
        $this->add_group_control(
            Group_Control_Border::get_type(), [
                'name'          => 'apcw_sale_badge_border',
                'selector'      => '{{WRAPPER}} .apcw-item-container .apcw-ribbon',
                'conditions'    => [
                    'terms'     => [
                        [
                            'name'      => 'apcw_product_grid_layout',
                            'operator'  => 'in',
                            'value'     => ['grid-layout-5'],
                        ],
                        [
                            'name'      => 'apcw_product_carousel_layout',
                            'operator'  => 'in',
                            'value'     => ['carousel-layout-5'],
                        ],
                    ],
                ],
            ]
        );

        $this->end_controls_section();
        //Sale Badge Style.

        //Start Pagination Style.
        $this->start_controls_section(
            'apcw_pagination_style', [
                'label'         => esc_html__('Pagination', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_STYLE,
                'condition'     => ['apcw_design' => 'carousel'],
            ]
        );

        //Pagination Color.
        $this->add_control(
			'apcw_pagination_color', [
				'label'         => esc_html__('Pagination Color', APCW_DOMAIN),
				'type'          => Controls_Manager::COLOR,
                'default'       => '#5068D842',
				'selectors'     => [
					'{{WRAPPER}} .carousel-layout-1 .slick-dots li button::before,
                    {{WRAPPER}} .carousel-layout-2 .slick-dots li button::before' => 'color: {{VALUE}} !important;',
                    '{{WRAPPER}} .carousel-layout-3 .slick-dots li button::before,
                    {{WRAPPER}} .carousel-layout-4 .slick-dots li button::before,
                    {{WRAPPER}} .carousel-layout-4 .slick-dots li.slick-active button::before' => 'border-color: {{VALUE}} !important;',
                    '{{WRAPPER}} .carousel-layout-5 .slick-dots li button::before' => 'background-color: {{VALUE}} !important;',
				],
			]
		);

        //Active Pagination Color.
        $this->add_control(
			'apcw_active_pagination_color', [
				'label'         => esc_html__('Active Pagination Color', APCW_DOMAIN),
				'type'          => Controls_Manager::COLOR,
                'default'       => '#5068D8',
				'selectors'     => [
					'{{WRAPPER}} .carousel-layout-1 .slick-dots li.slick-active button::before,
                    {{WRAPPER}} .carousel-layout-2 .slick-dots li.slick-active button::before' => 'color: {{VALUE}} !important;',
                    '{{WRAPPER}} .carousel-layout-3 .slick-dots li.slick-active button::before' => 'border-color: {{VALUE}} !important;',
                    '{{WRAPPER}} .carousel-layout-4 .slick-dots li.slick-active button::before,
                    {{WRAPPER}} .carousel-layout-5 .slick-dots li.slick-active button::before' => 'background-color: {{VALUE}} !important;',
				],
			]
		);

        //Pagination Size.
        $this->add_control(
			'apcw_pagination_size', [
				'label'         => esc_html__('Pagination Size', APCW_DOMAIN),
				'type'          => Controls_Manager::SLIDER,
				'size_units'    => [ 'px', '%' ],
				'range'         => [
					'px'        => [
						'min'   => 0,
						'max'   => 100,
						'step'  => 1,
					],
				],
				'selectors'     => [
					'{{WRAPPER}} .carousel-layout-1 .slick-dots li button::before,
                    {{WRAPPER}} .carousel-layout-2 .slick-dots li button::before' => 'font-size: {{SIZE}}{{UNIT}};',
				],
                'condition'     => ['apcw_product_carousel_layout' => ['carousel-layout-1','carousel-layout-2'],],
			]
		);

        //Pagination Size.
        $this->add_control(
			'apcw_paginations_size', [
				'label'         => esc_html__('Pagination Size', APCW_DOMAIN),
				'type'          => Controls_Manager::SLIDER,
				'size_units'    => [ 'px', '%', 'custom' ],
				'range'         => [
					'px'        => [
						'min'   => 0,
						'max'   => 100,
						'step'  => 1,
					],
				],
				'selectors'     => [
					'{{WRAPPER}} .carousel-layout-4 .slick-dots li button::before,
                    {{WRAPPER}} .carousel-layout-5 .slick-dots li button::before' => 'height: {{SIZE}}{{UNIT}};  width: {{SIZE}}{{UNIT}};',
				],
                'condition'     => ['apcw_product_carousel_layout' => ['carousel-layout-4','carousel-layout-5'],],
			]
		);

        $this->end_controls_section();
        //End Pagination Style.

        //Start Navigation Style. 
        $this->start_controls_section(
            'apcw_arrow_style', [
                'label'         => esc_html__('Arrows', APCW_DOMAIN),
                'tab'           => Controls_Manager::TAB_STYLE,
                'condition'     => ['apcw_design' => 'carousel'],
            ]
        );

        //Arrow Size.
        $this->add_control(
			'apcw_arrow_size', [
				'label'         => esc_html__('Arrow Size', APCW_DOMAIN),
				'type'          => Controls_Manager::SLIDER,
				'size_units'    => [ 'px', '%' ],
				'range'         => [
					'px'        => [
						'min'   => 0,
						'max'   => 100,
						'step'  => 1,
					],
				],
                'default'       => [
                    'size'      => 24,
					'unit'      => 'px',
                ],
				'selectors'     => [
					'{{WRAPPER}} .apcw-product-carousel .slick-prev i,
                    {{WRAPPER}} .apcw-product-carousel .slick-next i' => 'font-size: {{SIZE}}{{UNIT}};',
                    '{{WRAPPER}} .apcw-product-carousel .slick-prev svg,
                    {{WRAPPER}} .apcw-product-carousel .slick-next svg' => 'width:{{SIZE}}{{UNIT}}; height:{{SIZE}}{{UNIT}};',
				],
			]
		);

        //Arrow Color.
        $this->add_control(
			'apcw_arrow_color', [
				'label'         => esc_html__('Arrow Color', APCW_DOMAIN),
				'type'          => Controls_Manager::COLOR,
                'default'       => '#B6B6B6',
				'selectors'     => [
					'{{WRAPPER}} .apcw-product-carousel .slick-prev i,
                    {{WRAPPER}} .apcw-product-carousel .slick-next i' => 'color: {{VALUE}} !important;',
                    '{{WRAPPER}} .apcw-product-carousel .slick-prev svg,
                    {{WRAPPER}} .apcw-product-carousel .slick-next svg' => 'fill:{{VALUE}}',
				],
			]
		);

        //Arrow Background Color.
        $this->add_control(
			'apcw_arrow_background_color', [
				'label'         => esc_html__('Arrow Background Color', APCW_DOMAIN),
				'type'          => Controls_Manager::COLOR,
				'selectors'     => [
					'{{WRAPPER}} .apcw-product-carousel .slick-prev i,
                    {{WRAPPER}} .apcw-product-carousel .slick-next i' => 'background-color: {{VALUE}} !important;',
				],
			]
		);

        //Arrow Border Radius.
        $this->add_responsive_control(
            'apcw_arrow_border_radius', [
                'label'         => esc_html__('Arrow Border Radius', APCW_DOMAIN),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => [ 'px', '%' ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-product-carousel .slick-prev i,
                    {{WRAPPER}} .apcw-product-carousel .slick-next i' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        //Arrow Padding.
        $this->add_responsive_control(
            'apcw_arrow_padding', [
                'label'         => esc_html__('Arrow Padding', APCW_DOMAIN),
                'type'          => Controls_Manager::DIMENSIONS,
                'size_units'    => [ 'px', '%' ],
                'selectors'     => [
                    '{{WRAPPER}} .apcw-product-carousel .slick-prev i,
                    {{WRAPPER}} .apcw-product-carousel .slick-next i' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
                ],
            ]
        );

        $this->end_controls_section();
        //End Navigation Style.

    }

    /**
     * Render Item List Elements widget output on the frontend.
     */
    protected function render() {

        $settings = $this->get_settings_for_display();

        $product_style = $settings['apcw_product_grid_layout'];
        $apcw_slider_slides_to_show = $settings['apcw_slider_slides_to_show'];
        $apcw_slider_slides_to_show_tablet   =  $settings['apcw_slider_slides_to_show_tablet'];
        $apcw_slider_slides_to_show_mobile  = $settings['apcw_slider_slides_to_show_mobile'];

        if ($settings['apcw_grid_columns'] == "1") {
            $column = "elementor-grid-1";
        } elseif ($settings['apcw_grid_columns'] == "2") {
            $column = "elementor-grid-2";
        } elseif ($settings['apcw_grid_columns'] == "3") {
            $column = "elementor-grid-3";
        } elseif ($settings['apcw_grid_columns'] == "4") {
            $column = "elementor-grid-4";
        } elseif ($settings['apcw_grid_columns'] == "5") {
            $column = "elementor-grid-5";
        } elseif ($settings['apcw_grid_columns'] == "6") {
            $column = "elementor-grid-6";
        }

        $args = array(
            'post_type'         => 'product',
            'posts_per_page'    => $settings['apcw_number_of_posts'],
            'post_status'       => 'publish',
            'orderby'           => $settings['apcw_product_orderby'],
            'order'             => $settings['apcw_product_sort_order'],
            'offset'            => $settings['apcw_product_offset'],
        );

        //Product Type.
        if( isset($settings['apcw_product_type']) && !empty($settings['apcw_product_type']) ){
			$args['product_type'] = $settings['apcw_product_type'];
		}

        //Product post per page.
        if (isset($settings['apcw_number_of_posts']) && !empty($settings['apcw_number_of_posts'])) {
            $args['posts_per_page'] = $settings['apcw_number_of_posts'];
        } else {
            $args['posts_per_page'] = -1;
        }

        //Filter By Product Source.
        $args['meta_query'] = ['relation' => 'AND'];

        if ($settings['apcw_product_source'] == 'featured-products') {
            $args['tax_query'] = [
                'relation'  => 'AND',
                [
                    'taxonomy'  => 'product_visibility',
                    'field'     => 'name',
                    'terms'     => 'featured',
                ],
                [
                    'taxonomy'  => 'product_visibility',
                    'field'     => 'name',
                    'terms'     => ['exclude-from-search', 'exclude-from-catalog'],
                    'operator'  => 'NOT IN',
                ],
            ];
        } else if ($settings['apcw_product_source'] == 'best-selling-products') {
            $args['meta_key']   = 'total_sales';
            $args['orderby']    = 'meta_value_num';
            $args['order']      = 'DESC';
        } else if ($settings['apcw_product_source'] == 'sale-products') {
            $args['post__in']   = array_merge( array( 0 ), wc_get_product_ids_on_sale() );
        } else if ($settings['apcw_product_source'] == 'top-products') {
            $args['meta_key']   = '_wc_average_rating';
            $args['orderby']    = 'meta_value_num';
            $args['order']  = 'DESC';
        }

        //Exclude IN or NOT IN.
        if (isset($settings['apcw_exclude_product_categories']) && !empty($settings['apcw_exclude_product_categories'])) {
            $apcw_product_cats = 'NOT IN';
        } else {
            $apcw_product_cats = 'IN';
        }
        if (isset($settings['apcw_exclude_product_tags']) && !empty($settings['apcw_exclude_product_tags'])) {
            $apcw_product_tags = 'NOT IN';
        } else {
            $apcw_product_tags = 'IN';
        }
        if (isset($settings['apcw_exclude_product_authors']) && !empty($settings['apcw_exclude_product_authors'])) {
            $apcw_product_author = 'author__not_in';
        } else {
            $apcw_product_author = 'author__in';
        }

    //Query Check all Display Category, tags and Author .
    if (( isset($settings['apcw_product_categories']) && !empty($settings['apcw_product_categories']) ) && ( isset($settings['apcw_product_tags']) && !empty($settings['apcw_product_tags']) ) && ( isset($settings['product_author']) && !empty($settings['product_author']) )) {
        //Display All Post with Category, tags and Author //Condition AND.
        $args['tax_query'][] = array(
            'relation'      => 'AND',
            array(
                'taxonomy'  => 'product_cat',
                'field'     => 'ID',
                'terms'     => $settings['apcw_product_categories'],
                'operator'  => $apcw_product_cats,
            ),
            array(
                'taxonomy'  => 'product_tag',
                'field'     => 'ID',
                'terms'     => $settings['apcw_product_tags'],
                'operator'  => $apcw_product_tags,
            ),
        );
        $args[$apcw_product_author] = $settings['apcw_product_author'];
    } elseif ((isset($settings['apcw_product_categories']) && !empty($settings['apcw_product_categories'])) && (isset($settings['apcw_product_tags']) && !empty($settings['apcw_product_tags']))) {
        //Display All Post with Category and Tags //Condition AND.
        $args['tax_query'][] = array(
            'relation'      => 'AND',
            array(
                'taxonomy'  => 'product_cat',
                'field'     => 'ID',
                'terms'     => $settings['apcw_product_categories'],
                'operator'  => $apcw_product_cats,
            ),
            array(
                'taxonomy'  => 'product_tag',
                'field'     => 'ID',
                'terms'     => $settings['apcw_product_tags'],
                'operator'  => $apcw_product_tags,
            ),
        );
    } elseif ((isset($settings['apcw_product_categories']) && !empty($settings['apcw_product_categories'])) && (isset($settings['apcw_product_author']) && !empty($settings['apcw_product_author']))) {
        //Display All Post with Category and Author //Condition AND.            
        $args['tax_query'][] = array(
            'relation'      => 'AND',
            array(
                'taxonomy'  => 'product_cat',
                'field'     => 'ID',
                'terms'     => $settings['apcw_product_categories'],
                'operator'  => $apcw_product_cats,
            ),
        );
        $args[$apcw_product_author] = $settings['apcw_product_author'];
    } elseif ((isset($settings['apcw_product_tags']) && !empty($settings['apcw_product_tags'])) && (isset($settings['apcw_product_author']) && !empty($settings['apcw_product_author']))) {
        //Display All Post with Tags and Author //Condition AND.
        $args['tax_query'][] = array(
            'relation'      => 'AND',
            array(
                'taxonomy'  => 'product_tag',
                'field'     => 'ID',
                'terms'     => $settings['apcw_product_tags'],
                'operator'  => $apcw_product_tags,
            ),
        );
        $args[$apcw_product_author] = $settings['apcw_product_author'];
    } elseif (isset($settings['apcw_product_categories']) && !empty($settings['apcw_product_categories'])) {
        //Display All Post with Category.
        $args['tax_query'][] = array(
            array(
                'taxonomy'  => 'product_cat',
                'field'     => 'ID',
                'terms'     => $settings['apcw_product_categories'],
                'operator'  => $apcw_product_cats,
            ),
        );
    } elseif (isset($settings['apcw_product_tags']) && !empty($settings['apcw_product_tags'])) {
        //Display All Post with Tags. 
        $args['tax_query'][] = array(
            array(
                'taxonomy'  => 'product_tag',
                'field'     => 'ID',
                'terms'     => $settings['apcw_product_tags'],
                'operator'  => $apcw_product_tags,
            ),
        );
    } elseif (isset($settings['apcw_product_author']) && !empty($settings['apcw_product_author'])) {
        //Display All Post with Authors.
        $args[$apcw_product_author] = $settings['apcw_product_author'];
    }

    $products = new \WP_Query($args);

    if ($products->have_posts()) :  

        $this->add_render_attribute(
            'apcw_slider_setting', [
                'id'                                => 'apcw-slider-' . $this->get_id(),
                'data-apcw-slides-to-show'          => $apcw_slider_slides_to_show,
                'data-apcw-slides-to-show-tablet'   => $apcw_slider_slides_to_show_tablet,
                'data-apcw-slides-to-show-mobile'   => $apcw_slider_slides_to_show_mobile,
                'data-apcw-slider-autoplay'         => $settings['apcw_slider_autoplay'],
                'data-apcw-slider-autoplay-speed'   => $settings['apcw_slider_speed'],
                'data-apcw-slider-dots'             => $settings['apcw_slider_dots'],
                'data-apcw-slider-arrows'           => $settings['apcw_slider_arrows'],
                'data-apcw-slider-to-scroll'        => $settings['apcw_slider_slides_to_scroll'],
                'data-apcw-slider-infinite'         => $settings['apcw_slider_infinite'],   
                'data-apcw-slider-pause-on-hover'   => $settings['apcw_slider_pause_on_hover'], 
                'data-apcw-prev-arrow' => Sliders_Icons_managers::render_icon($settings['apcw_prev_icon'], [ 'aria-hidden' => 'true' ]),
                'data-apcw-next-arrow' => Sliders_Icons_managers::render_icon($settings['apcw_next_icon'], [ 'aria-hidden' => 'true' ]),
            ]
        );
        $gridClass = "";
        $carouselClass = "";
        $getAttributes = "";
        if($settings['apcw_design'] == "carousel") {
            $carouselClass = "apcw-product-carousel";
            $getAttributes = $this->get_render_attribute_string('apcw_slider_setting');
        }

        if($settings['apcw_design'] == "grid") {
            $gridClass = "grid-container product-grid-gap elementor-grid ".$product_style . ' ' . $column."";
        }
        ?>
            <div class="advanced-product-catalog-woo <?php echo $settings['apcw_product_carousel_layout']. ' ' . $carouselClass. '' .$gridClass; ?>" <?php echo $getAttributes; ?>>
                <?php
                while ($products->have_posts()) : $products->the_post();

                    //Grid Layouts.
                    if($settings['apcw_design'] == "grid") {
                        switch ($settings['apcw_product_grid_layout']) {
                            case 'grid-layout-1':
                                include APCW_PATH . 'include/grid/layout-1.php';  //Grid Layout 1.
                                break;
                            case 'grid-layout-2':
                                include APCW_PATH . 'include/grid/layout-2.php';  //Grid Layout 2.
                                break;
                            case 'grid-layout-3':
                                include APCW_PATH . 'include/grid/layout-3.php';  //Grid Layout 3.
                                break;
                            case 'grid-layout-4':
                                include APCW_PATH . 'include/grid/layout-4.php';  //Grid Layout 4.
                                break;
                            case 'grid-layout-5':
                                include APCW_PATH . 'include/grid/layout-5.php';  //Grid Layout 5.
                                break;
                            default:
                                include APCW_PATH . 'include/grid/layout-1.php';  //Grid Layout 1.
                                break;
                        }
                    }

                    //Carousel Layouts.
                    if($settings['apcw_design'] == "carousel") {
                        switch ($settings['apcw_product_carousel_layout']) {
                            case 'carousel-layout-1':
                                include APCW_PATH . 'include/carousel/layout-1.php';  //Carousel Layout 1.
                                break;
                            case 'carousel-layout-2':
                                include APCW_PATH . 'include/carousel/layout-2.php';  //Carousel Layout 2.
                                break;
                            case 'carousel-layout-3':
                                include APCW_PATH . 'include/carousel/layout-3.php';  //Carousel Layout 3.
                                break;
                            case 'carousel-layout-4':
                                include APCW_PATH . 'include/carousel/layout-4.php';  //Carousel Layout 4.
                                break;
                            case 'carousel-layout-5':
                                include APCW_PATH . 'include/carousel/layout-5.php';  //Carousel Layout 5.
                                break;
                            default:
                                include APCW_PATH . 'include/carousel/layout-1.php';  //Carousel Layout 1.
                                break;
                        }
                    }

                endwhile;
                wp_reset_postdata();
                ?>
            </div> 
            <?php
        endif;
    }
}

Plugin::instance()->widgets_manager->register(new APCW_Widget());   